import mongoose, { Model, ClientSession, FilterQuery, HydratedDocument, UpdateQuery, SaveOptions, QueryOptions, Document, UpdateWithAggregationPipeline, PipelineStage } from 'mongoose';
import { AggregateOptions, UpdateResult, CollationOptions } from 'mongodb';
interface DatabaseService {
    findById<DocType>(collRef: Model<DocType>, id: string, projection?: Record<string, any>, options?: QueryOptions, session?: ClientSession): Promise<mongoose.HydratedDocument<DocType> | null>;
    findOne<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, projection?: Record<string, any>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
    find<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, projection?: Record<string, any>, options?: QueryOptions, sort?: Record<string, number>, skip?: number, limit?: number, session?: ClientSession, collationOptions?: CollationOptions): Promise<HydratedDocument<DocType>[]>;
    save<DocType>(docRef: HydratedDocument<DocType>, options?: SaveOptions): Promise<DocType | Document<unknown, any, DocType> & {
        _id: mongoose.Types.ObjectId;
    }>;
    findByIdAndUpdate<DocType>(collRef: Model<DocType>, id: string, updateQuery: UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
    findOneAndUpdate<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, updateQuery: UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
    updateOne<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, updateQuery: UpdateWithAggregationPipeline | UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<UpdateResult>;
    updateMany<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, updateQuery: UpdateWithAggregationPipeline | UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<UpdateResult>;
    aggregate<DocType>(collRef: Model<DocType>, pipeline?: PipelineStage[], options?: AggregateOptions, session?: ClientSession): Promise<any[]>;
    countDocuments<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<number>;
    insertMany<DocType>(collRef: Model<DocType>, docs: DocType[], options?: mongoose.InsertManyOptions & {
        rawResult: true;
    }): Promise<mongoose.HydratedDocument<DocType, {}, {}>[]>;
    findOneAndDelete<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
}
declare class MongooseService implements DatabaseService {
    private static _instance;
    private constructor();
    static getInstance(): MongooseService;
    private _reconnectIfDisconnected;
    /**
     * Get a mongoose document by id
     * @param collRef
     * @param id
     * @param projection
     * @param options
     * @param session
     * @returns
     */
    findById<DocType>(collRef: Model<DocType>, id: string, projection?: Record<string, any>, options?: QueryOptions, session?: ClientSession): Promise<mongoose.HydratedDocument<DocType> | null>;
    /**
     * Get a mongoose document for the requested filter query
     * @param collRef
     * @param query
     * @param projection
     * @param options
     * @param session
     * @returns
     */
    findOne<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, projection?: Record<string, any>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
    /**
     * Get all matched mongoose documents for the requested filter query
     * @param collRef
     * @param query
     * @param projection
     * @param options
     * @param sort
     * @param skip
     * @param limit
     * @param session
     * @returns
     */
    find<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, projection?: Record<string, any>, options?: QueryOptions, sort?: Record<string, number>, skip?: number, limit?: number, session?: ClientSession, collationOptions?: CollationOptions, distinct?: string): Promise<HydratedDocument<DocType>[]>;
    /**
     * Saving a mongoose document
     * @param docRef
     * @param options
     * @returns
     */
    save<DocType>(docRef: HydratedDocument<DocType>, options?: SaveOptions): Promise<DocType | Document<unknown, any, DocType> & {
        _id: mongoose.Types.ObjectId;
    }>;
    /**
     * Finding a document by an id and updating its info.
     * @param collRef
     * @param id
     * @param updateQuery
     * @param options
     * @param session
     * @returns
     */
    findByIdAndUpdate<DocType>(collRef: Model<DocType>, id: string, updateQuery: UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
    /**
     * Finding a document by some key and updating its info.
     * @param collRef
     * @param query
     * @param updateQuery
     * @param options
     * @param session
     * @returns
     */
    findOneAndUpdate<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, updateQuery: UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
    /**
     * Updating a document of a collection.
     * @param collRef
     * @param query
     * @param updateQuery
     * @param options
     * @param session
     * @returns
     */
    updateOne<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, updateQuery: UpdateWithAggregationPipeline | UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<UpdateResult>;
    /**
     * Updating many documents of a collection.
     * @param collRef
     * @param query
     * @param updateQuery
     * @param options
     * @param session
     * @returns
     */
    updateMany<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, updateQuery: UpdateWithAggregationPipeline | UpdateQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<UpdateResult>;
    /**
     * Mongoose aggregation queries
     * @param collRef
     * @param pipeline
     * @param options
     * @returns
     */
    aggregate<DocType>(collRef: Model<DocType>, pipeline?: PipelineStage[], options?: AggregateOptions, session?: ClientSession, ignoreSoftDelete?: boolean): Promise<any[]>;
    /**
     * Counts the number of documents that matches the query
     * @param collRef
     * @param query
     * @param options
     * @param session
     * @returns
     */
    countDocuments<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<number>;
    insertMany<DocType>(collRef: Model<DocType>, docs: DocType[], options?: mongoose.InsertManyOptions & {
        rawResult: true;
    }): Promise<mongoose.HydratedDocument<DocType, {}, {}>[]>;
    findOneAndDelete<DocType>(collRef: Model<DocType>, query: FilterQuery<DocType>, options?: QueryOptions, session?: ClientSession): Promise<HydratedDocument<DocType> | null>;
}
export { DatabaseService, MongooseService };
